@@ -19,7 +19,6 @@ else:
     import unittest
 
 
-
 class TestParse(unittest.TestCase):
 
     def test_simple_counter(self):
@@ -104,7 +103,8 @@ a_count 3
 a_sum 2
 # EOF
 """)
-        self.assertEqual([HistogramMetricFamily("a", "help", sum_value=2, buckets=[("1.0", 0.0), ("+Inf", 3.0)])], list(families))
+        self.assertEqual([HistogramMetricFamily("a", "help", sum_value=2, buckets=[("1.0", 0.0), ("+Inf", 3.0)])],
+                         list(families))
 
     def test_histogram_exemplars(self):
         families = text_string_to_metric_families("""# TYPE a histogram
@@ -117,7 +117,9 @@ a_bucket{le="+Inf"} 3 # {a="1234567890123456789012345678901234567890123456789012
         hfm = HistogramMetricFamily("a", "help")
         hfm.add_sample("a_bucket", {"le": "1.0"}, 0.0, None, Exemplar({"a": "b"}, 0.5))
         hfm.add_sample("a_bucket", {"le": "2.0"}, 2.0, None, Exemplar({"a": "c"}, 0.5)),
-        hfm.add_sample("a_bucket", {"le": "+Inf"}, 3.0, None, Exemplar({"a": "1234567890123456789012345678901234567890123456789012345678"}, 4, Timestamp(123, 0)))
+        hfm.add_sample("a_bucket", {"le": "+Inf"}, 3.0, None,
+                       Exemplar({"a": "1234567890123456789012345678901234567890123456789012345678"}, 4,
+                                Timestamp(123, 0)))
         self.assertEqual([hfm], list(families))
 
     def test_simple_gaugehistogram(self):
@@ -129,7 +131,8 @@ a_gcount 3
 a_gsum 2
 # EOF
 """)
-        self.assertEqual([GaugeHistogramMetricFamily("a", "help", gsum_value=2, buckets=[("1.0", 0.0), ("+Inf", 3.0)])], list(families))
+        self.assertEqual([GaugeHistogramMetricFamily("a", "help", gsum_value=2, buckets=[("1.0", 0.0), ("+Inf", 3.0)])],
+                         list(families))
 
     def test_gaugehistogram_exemplars(self):
         families = text_string_to_metric_families("""# TYPE a gaugehistogram
@@ -293,17 +296,9 @@ a_total{foo=""} 2
         self.assertEqual([metric_family], list(families))
 
     def test_label_escaping(self):
-        for escaped_val, unescaped_val in [
-                ('foo', 'foo'),
-                ('\\foo', '\\foo'),
-                ('\\\\foo', '\\foo'),
-                ('foo\\\\', 'foo\\'),
-                ('\\\\', '\\'),
-                ('\\n', '\n'),
-                ('\\\\n', '\\n'),
-                ('\\\\\\n', '\\\n'),
-                ('\\"', '"'),
-                ('\\\\\\"', '\\"')]:
+        for escaped_val, unescaped_val in [('foo', 'foo'), ('\\foo', '\\foo'), ('\\\\foo', '\\foo'),
+                                           ('foo\\\\', 'foo\\'), ('\\\\', '\\'), ('\\n', '\n'),
+                                           ('\\\\n', '\\n'), ('\\\\\\n', '\\\n'), ('\\"', '"'), ('\\\\\\"', '\\"')]:
             families = list(text_string_to_metric_families("""# TYPE a counter
 # HELP a help
 a_total{foo="%s",bar="baz"} 1
@@ -379,6 +374,93 @@ b_total 2 1234567890
         b.add_metric([], 2, timestamp=Timestamp(1234567890, 0))
         self.assertEqual([a, b], list(families))
 
+    def test_hash_in_label_value(self):
+        families = text_string_to_metric_families("""# TYPE a counter
+# HELP a help
+a_total{foo="foo # bar"} 1
+a_total{foo="} foo # bar # "} 1
+# EOF
+""")
+        a = CounterMetricFamily("a", "help", labels=["foo"])
+        a.add_metric(["foo # bar"], 1)
+        a.add_metric(["} foo # bar # "], 1)
+        self.assertEqual([a], list(families))
+
+    def test_exemplars_with_hash_in_label_values(self):
+        families = text_string_to_metric_families("""# TYPE a histogram
+# HELP a help
+a_bucket{le="1.0",foo="bar # "} 0 # {a="b",foo="bar # bar"} 0.5
+a_bucket{le="2.0",foo="bar # "} 2 # {a="c",foo="bar # bar"} 0.5
+a_bucket{le="+Inf",foo="bar # "} 3 # {a="d",foo="bar # bar"} 4
+# EOF
+""")
+        hfm = HistogramMetricFamily("a", "help")
+        hfm.add_sample("a_bucket", {"le": "1.0", "foo": "bar # "}, 0.0, None, Exemplar({"a": "b", "foo": "bar # bar"}, 0.5))
+        hfm.add_sample("a_bucket", {"le": "2.0", "foo": "bar # "}, 2.0, None, Exemplar({"a": "c", "foo": "bar # bar"}, 0.5))
+        hfm.add_sample("a_bucket", {"le": "+Inf", "foo": "bar # "}, 3.0, None, Exemplar({"a": "d", "foo": "bar # bar"}, 4))
+        self.assertEqual([hfm], list(families))
+
+    @unittest.skipIf(sys.version_info < (3, 3), "Test requires Python 3.3+.")
+    def test_fallback_to_state_machine_label_parsing(self):
+        from unittest.mock import patch
+        from prometheus_client.openmetrics.parser import _parse_sample
+
+        parse_sample_function = "prometheus_client.openmetrics.parser._parse_sample"
+        parse_labels_function = "prometheus_client.openmetrics.parser._parse_labels"
+        parse_remaining_function = "prometheus_client.openmetrics.parser._parse_remaining_text"
+        state_machine_function = "prometheus_client.openmetrics.parser._parse_labels_with_state_machine"
+
+        parse_sample_return_value = Sample("a_total", {"foo": "foo # bar"}, 1)
+        with patch(parse_sample_function, return_value=parse_sample_return_value) as mock:
+            families = text_string_to_metric_families("""# TYPE a counter
+# HELP a help
+a_total{foo="foo # bar"} 1
+# EOF
+""")
+            a = CounterMetricFamily("a", "help", labels=["foo"])
+            a.add_metric(["foo # bar"], 1)
+            self.assertEqual([a], list(families))
+            mock.assert_called_once_with('a_total{foo="foo # bar"} 1')
+
+        # First fallback case
+        state_machine_return_values = [{"foo": "foo # bar"}, len('foo="foo # bar"}')]
+        parse_remaining_values = [1, None, None]
+        with patch(parse_labels_function) as mock1:
+            with patch(state_machine_function, return_value=state_machine_return_values) as mock2:
+                with patch(parse_remaining_function, return_value=parse_remaining_values) as mock3:
+                    sample = _parse_sample('a_total{foo="foo # bar"} 1')
+                    s = Sample("a_total", {"foo": "foo # bar"}, 1)
+                    self.assertEqual(s, sample)
+                    mock1.assert_not_called()
+                    mock2.assert_called_once_with('foo="foo # bar"} 1')
+                    mock3.assert_called_once_with('1')
+
+        # Second fallback case
+        state_machine_return_values = [{"le": "1.0"}, len('le="1.0"}')]
+        parse_remaining_values = [0.0, Timestamp(123, 0), Exemplar({"a": "b"}, 0.5)]
+        with patch(parse_labels_function) as mock1:
+            with patch(state_machine_function, return_value=state_machine_return_values) as mock2:
+                with patch(parse_remaining_function, return_value=parse_remaining_values) as mock3:
+                    sample = _parse_sample('a_bucket{le="1.0"} 0 123 # {a="b"} 0.5')
+                    s = Sample("a_bucket", {"le": "1.0"}, 0.0, Timestamp(123, 0), Exemplar({"a": "b"}, 0.5))
+                    self.assertEqual(s, sample)
+                    mock1.assert_not_called()
+                    mock2.assert_called_once_with('le="1.0"} 0 123 # {a="b"} 0.5')
+                    mock3.assert_called_once_with('0 123 # {a="b"} 0.5')
+
+        # No need to fallback case
+        parse_labels_return_values = {"foo": "foo#bar"}
+        parse_remaining_values = [1, None, None]
+        with patch(parse_labels_function, return_value=parse_labels_return_values) as mock1:
+            with patch(state_machine_function) as mock2:
+                with patch(parse_remaining_function, return_value=parse_remaining_values) as mock3:
+                    sample = _parse_sample('a_total{foo="foo#bar"} 1')
+                    s = Sample("a_total", {"foo": "foo#bar"}, 1)
+                    self.assertEqual(s, sample)
+                    mock1.assert_called_once_with('foo="foo#bar"')
+                    mock2.assert_not_called()
+                    mock3.assert_called_once_with('1')
+
     @unittest.skipIf(sys.version_info < (2, 7), "Test requires Python 2.7+.")
     def test_roundtrip(self):
         text = """# HELP go_gc_duration_seconds A summary of the GC invocation durations.
@@ -449,141 +531,152 @@ foo_created 1.520430000123e+09
 
     def test_invalid_input(self):
         for case in [
-                # No EOF.
-                (''),
-                # Text after EOF.
-                ('a 1\n# EOF\nblah'),
-                ('a 1\n# EOFblah'),
-                # Missing or wrong quotes on label value.
-                ('a{a=1} 1\n# EOF\n'),
-                ('a{a="1} 1\n# EOF\n'),
-                ('a{a=\'1\'} 1\n# EOF\n'),
-                # Missing or extra commas.
-                ('a{a="1"b="2"} 1\n# EOF\n'),
-                ('a{a="1",,b="2"} 1\n# EOF\n'),
-                ('a{a="1",b="2",} 1\n# EOF\n'),
-                # Missing value.
-                ('a\n# EOF\n'),
-                ('a \n# EOF\n'),
-                # Bad HELP.
-                ('# HELP\n# EOF\n'),
-                ('# HELP \n# EOF\n'),
-                ('# HELP a\n# EOF\n'),
-                ('# HELP a\t\n# EOF\n'),
-                (' # HELP a meh\n# EOF\n'),
-                # Bad TYPE.
-                ('# TYPE\n# EOF\n'),
-                ('# TYPE \n# EOF\n'),
-                ('# TYPE a\n# EOF\n'),
-                ('# TYPE a\t\n# EOF\n'),
-                ('# TYPE a meh\n# EOF\n'),
-                ('# TYPE a meh \n# EOF\n'),
-                ('# TYPE a gauge \n# EOF\n'),
-                ('# TYPE a untyped\n# EOF\n'),
-                # Bad UNIT.
-                ('# UNIT\n# EOF\n'),
-                ('# UNIT \n# EOF\n'),
-                ('# UNIT a\n# EOF\n'),
-                ('# UNIT a\t\n# EOF\n'),
-                ('# UNIT a seconds\n# EOF\n'),
-                ('# UNIT a_seconds seconds \n# EOF\n'),
-                ('# TYPE x_u info\n# UNIT x_u u\n# EOF\n'),
-                ('# TYPE x_u stateset\n# UNIT x_u u\n# EOF\n'),
-                # Metadata in wrong place.
-                ('# HELP a x\na 1\n# TYPE a gauge\n# EOF\n'),
-                ('# TYPE a gauge\na 1\n# HELP a gauge\n# EOF\n'),
-                ('# TYPE a_s gauge\na_s 1\n# UNIT a_s s\n# EOF\n'),
-                # Repeated metadata.
-                ('# HELP a \n# HELP a \n# EOF\n'),
-                ('# HELP a x\n# HELP a x\n# EOF\n'),
-                ('# TYPE a untyped\n# TYPE a untyped\n# EOF\n'),
-                ('# UNIT a_s s\n# UNIT a_s s\n# EOF\n'),
-                # Bad metric names.
-                ('0a 1\n# EOF\n'),
-                ('a.b 1\n# EOF\n'),
-                ('a-b 1\n# EOF\n'),
-                # Bad value.
-                ('a a\n# EOF\n'),
-                ('a  1\n# EOF\n'),
-                ('a 1\t\n# EOF\n'),
-                ('a 1 \n# EOF\n'),
-                # Bad timestamp.
-                ('a 1 z\n# EOF\n'),
-                ('a 1 1z\n# EOF\n'),
-                ('a 1 1.1.1\n# EOF\n'),
-                ('a 1 NaN\n# EOF\n'),
-                ('a 1 Inf\n# EOF\n'),
-                ('a 1 +Inf\n# EOF\n'),
-                ('a 1 -Inf\n# EOF\n'),
-                # Bad exemplars.
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 #\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1# {} 1\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 #{} 1\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {}1\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {} 1 \n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {} 1 1 \n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {a="12345678901234567890123456789012345678901234567890123456789"} 1 1\n# EOF\n'),
-                # Exemplars on unallowed samples.
-                ('# TYPE a histogram\na_sum 1 # {a="b"} 0.5\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_sum 1 # {a="b"} 0.5\n# EOF\n'),
-                ('# TYPE a_bucket gauge\na_bucket 1 # {a="b"} 0.5\n# EOF\n'),
-                # Bad stateset/info values.
-                ('# TYPE a stateset\na 2\n# EOF\n'),
-                ('# TYPE a info\na 2\n# EOF\n'),
-                ('# TYPE a stateset\na 2.0\n# EOF\n'),
-                ('# TYPE a info\na 2.0\n# EOF\n'),
-                # Missing or invalid labels for a type.
-                ('# TYPE a summary\na 0\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="-1"} 0\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="foo"} 0\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="1.01"} 0\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="NaN"} 0\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="1"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket 0\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket 0\n# EOF\n'),
-                ('# TYPE a stateset\na 0\n# EOF\n'),
-                # Bad counter values.
-                ('# TYPE a counter\na_total NaN\n# EOF\n'),
-                ('# TYPE a counter\na_total -1\n# EOF\n'),
-                ('# TYPE a histogram\na_sum NaN\n# EOF\n'),
-                ('# TYPE a histogram\na_count NaN\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} NaN\n# EOF\n'),
-                ('# TYPE a histogram\na_sum -1\n# EOF\n'),
-                ('# TYPE a histogram\na_count -1\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} -1\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} NaN\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} -1\na_gcount -1\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} -1\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} 1\na_gsum -1\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} 1\na_gsum NaN\n# EOF\n'),
-                ('# TYPE a summary\na_sum NaN\n# EOF\n'),
-                ('# TYPE a summary\na_count NaN\n# EOF\n'),
-                ('# TYPE a summary\na_sum -1\n# EOF\n'),
-                ('# TYPE a summary\na_count -1\n# EOF\n'),
-                ('# TYPE a summary\na{quantile="0.5"} -1\n# EOF\n'),
-                # Bad histograms.
-                ('# TYPE a histogram\na_sum 1\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_gsum 1\n# EOF\n'),
-                ('# TYPE a histogram\na_count 1\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+Inf"} 0\na_count 1\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="1"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="1e-04"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="1e+05"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="+INF"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="2"} 0\na_bucket{le="1"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="1"} 1\na_bucket{le="2"} 1\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                # Bad grouping or ordering.
-                ('# TYPE a histogram\na_sum{a="1"} 0\na_sum{a="2"} 0\na_count{a="1"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{a="1",le="1"} 0\na_bucket{a="2",le="+Inf""} 0\na_bucket{a="1",le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a gaugehistogram\na_gsum{a="1"} 0\na_gsum{a="2"} 0\na_gcount{a="1"} 0\n# EOF\n'),
-                ('# TYPE a summary\nquantile{quantile="0"} 0\na_sum{a="1"} 0\nquantile{quantile="1"} 0\n# EOF\n'),
-                ('# TYPE a gauge\na 0 -1\na 0 -2\n# EOF\n'),
-                ('# TYPE a gauge\na 0 -1\na 0 -1.1\n# EOF\n'),
-                ('# TYPE a gauge\na 0 1\na 0 -1\n# EOF\n'),
-                ('# TYPE a gauge\na 0 1.1\na 0 1\n# EOF\n'),
-                ('# TYPE a gauge\na 0 1\na 0 0\n# EOF\n'),
-                ('# TYPE a gauge\na 0\na 0 0\n# EOF\n'),
-                ('# TYPE a gauge\na 0 0\na 0\n# EOF\n'),
+            # No EOF.
+            (''),
+            # Text after EOF.
+            ('a 1\n# EOF\nblah'),
+            ('a 1\n# EOFblah'),
+            # Missing or wrong quotes on label value.
+            ('a{a=1} 1\n# EOF\n'),
+            ('a{a="1} 1\n# EOF\n'),
+            ('a{a=\'1\'} 1\n# EOF\n'),
+            # Missing equal or label value.
+            ('a{a} 1\n# EOF\n'),
+            ('a{a"value"} 1\n# EOF\n'),
+            ('a{a""} 1\n# EOF\n'),
+            ('a{a=} 1\n# EOF\n'),
+            ('a{a="} 1\n# EOF\n'),
+            # Missing or extra commas.
+            ('a{a="1"b="2"} 1\n# EOF\n'),
+            ('a{a="1",,b="2"} 1\n# EOF\n'),
+            ('a{a="1",b="2",} 1\n# EOF\n'),
+            # Missing value.
+            ('a\n# EOF\n'),
+            ('a \n# EOF\n'),
+            # Bad HELP.
+            ('# HELP\n# EOF\n'),
+            ('# HELP \n# EOF\n'),
+            ('# HELP a\n# EOF\n'),
+            ('# HELP a\t\n# EOF\n'),
+            (' # HELP a meh\n# EOF\n'),
+            # Bad TYPE.
+            ('# TYPE\n# EOF\n'),
+            ('# TYPE \n# EOF\n'),
+            ('# TYPE a\n# EOF\n'),
+            ('# TYPE a\t\n# EOF\n'),
+            ('# TYPE a meh\n# EOF\n'),
+            ('# TYPE a meh \n# EOF\n'),
+            ('# TYPE a gauge \n# EOF\n'),
+            ('# TYPE a untyped\n# EOF\n'),
+            # Bad UNIT.
+            ('# UNIT\n# EOF\n'),
+            ('# UNIT \n# EOF\n'),
+            ('# UNIT a\n# EOF\n'),
+            ('# UNIT a\t\n# EOF\n'),
+            ('# UNIT a seconds\n# EOF\n'),
+            ('# UNIT a_seconds seconds \n# EOF\n'),
+            ('# TYPE x_u info\n# UNIT x_u u\n# EOF\n'),
+            ('# TYPE x_u stateset\n# UNIT x_u u\n# EOF\n'),
+            # Metadata in wrong place.
+            ('# HELP a x\na 1\n# TYPE a gauge\n# EOF\n'),
+            ('# TYPE a gauge\na 1\n# HELP a gauge\n# EOF\n'),
+            ('# TYPE a_s gauge\na_s 1\n# UNIT a_s s\n# EOF\n'),
+            # Repeated metadata.
+            ('# HELP a \n# HELP a \n# EOF\n'),
+            ('# HELP a x\n# HELP a x\n# EOF\n'),
+            ('# TYPE a untyped\n# TYPE a untyped\n# EOF\n'),
+            ('# UNIT a_s s\n# UNIT a_s s\n# EOF\n'),
+            # Bad metric names.
+            ('0a 1\n# EOF\n'),
+            ('a.b 1\n# EOF\n'),
+            ('a-b 1\n# EOF\n'),
+            # Bad value.
+            ('a a\n# EOF\n'),
+            ('a  1\n# EOF\n'),
+            ('a 1\t\n# EOF\n'),
+            ('a 1 \n# EOF\n'),
+            # Bad timestamp.
+            ('a 1 z\n# EOF\n'),
+            ('a 1 1z\n# EOF\n'),
+            ('a 1 1.1.1\n# EOF\n'),
+            ('a 1 NaN\n# EOF\n'),
+            ('a 1 Inf\n# EOF\n'),
+            ('a 1 +Inf\n# EOF\n'),
+            ('a 1 -Inf\n# EOF\n'),
+            # Bad exemplars.
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 #\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1# {} 1\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 #{} 1\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {}1\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {} 1 \n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # {} 1 1 \n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 1 # '
+             '{a="2345678901234567890123456789012345678901234567890123456789012345"} 1 1\n# EOF\n'),
+            # Exemplars on unallowed samples.
+            ('# TYPE a histogram\na_sum 1 # {a="b"} 0.5\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_sum 1 # {a="b"} 0.5\n# EOF\n'),
+            ('# TYPE a_bucket gauge\na_bucket 1 # {a="b"} 0.5\n# EOF\n'),
+            # Exemplars on unallowed metric types.
+            ('# TYPE a counter\na_total 1 # {a="b"} 1\n# EOF\n'),
+            ('# TYPE a gauge\na 1 # {a="b"} 1\n# EOF\n'),
+            # Bad stateset/info values.
+            ('# TYPE a stateset\na 2\n# EOF\n'),
+            ('# TYPE a info\na 2\n# EOF\n'),
+            ('# TYPE a stateset\na 2.0\n# EOF\n'),
+            ('# TYPE a info\na 2.0\n# EOF\n'),
+            # Missing or invalid labels for a type.
+            ('# TYPE a summary\na 0\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="-1"} 0\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="foo"} 0\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="1.01"} 0\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="NaN"} 0\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="1"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket 0\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket 0\n# EOF\n'),
+            ('# TYPE a stateset\na 0\n# EOF\n'),
+            # Bad counter values.
+            ('# TYPE a counter\na_total NaN\n# EOF\n'),
+            ('# TYPE a counter\na_total -1\n# EOF\n'),
+            ('# TYPE a histogram\na_sum NaN\n# EOF\n'),
+            ('# TYPE a histogram\na_count NaN\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} NaN\n# EOF\n'),
+            ('# TYPE a histogram\na_sum -1\n# EOF\n'),
+            ('# TYPE a histogram\na_count -1\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} -1\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} NaN\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} -1\na_gcount -1\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} -1\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} 1\na_gsum -1\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_bucket{le="+Inf"} 1\na_gsum NaN\n# EOF\n'),
+            ('# TYPE a summary\na_sum NaN\n# EOF\n'),
+            ('# TYPE a summary\na_count NaN\n# EOF\n'),
+            ('# TYPE a summary\na_sum -1\n# EOF\n'),
+            ('# TYPE a summary\na_count -1\n# EOF\n'),
+            ('# TYPE a summary\na{quantile="0.5"} -1\n# EOF\n'),
+            # Bad histograms.
+            ('# TYPE a histogram\na_sum 1\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_gsum 1\n# EOF\n'),
+            ('# TYPE a histogram\na_count 1\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+Inf"} 0\na_count 1\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="1"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="1e-04"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="1e+05"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="+INF"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="2"} 0\na_bucket{le="1"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="1"} 1\na_bucket{le="2"} 1\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            # Bad grouping or ordering.
+            ('# TYPE a histogram\na_sum{a="1"} 0\na_sum{a="2"} 0\na_count{a="1"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{a="1",le="1"} 0\na_bucket{a="2",le="+Inf""} '
+             '0\na_bucket{a="1",le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a gaugehistogram\na_gsum{a="1"} 0\na_gsum{a="2"} 0\na_gcount{a="1"} 0\n# EOF\n'),
+            ('# TYPE a summary\nquantile{quantile="0"} 0\na_sum{a="1"} 0\nquantile{quantile="1"} 0\n# EOF\n'),
+            ('# TYPE a gauge\na 0 -1\na 0 -2\n# EOF\n'),
+            ('# TYPE a gauge\na 0 -1\na 0 -1.1\n# EOF\n'),
+            ('# TYPE a gauge\na 0 1\na 0 -1\n# EOF\n'),
+            ('# TYPE a gauge\na 0 1.1\na 0 1\n# EOF\n'),
+            ('# TYPE a gauge\na 0 1\na 0 0\n# EOF\n'),
+            ('# TYPE a gauge\na 0\na 0 0\n# EOF\n'),
+            ('# TYPE a gauge\na 0 0\na 0\n# EOF\n'),
         ]:
             with self.assertRaises(ValueError):
                 list(text_string_to_metric_families(case))
@@ -591,9 +684,9 @@ foo_created 1.520430000123e+09
     @unittest.skipIf(sys.version_info < (2, 7), "float repr changed from 2.6 to 2.7")
     def test_invalid_float_input(self):
         for case in [
-                # Bad histograms.
-                ('# TYPE a histogram\na_bucket{le="9.999999999999999e+22"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
-                ('# TYPE a histogram\na_bucket{le="1.5555555555555201e+06"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            # Bad histograms.
+            ('# TYPE a histogram\na_bucket{le="9.999999999999999e+22"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
+            ('# TYPE a histogram\na_bucket{le="1.5555555555555201e+06"} 0\na_bucket{le="+Inf"} 0\n# EOF\n'),
         ]:
             with self.assertRaises(ValueError):
                 list(text_string_to_metric_families(case))
